package admin

import (
	"git.dmitriygnatenko.ru/dima/go-common/logger"
	"github.com/go-playground/validator/v10"
	"github.com/gofiber/fiber/v2"

	helper "git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/helpers/i18n"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/helpers/request"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/mapper"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/models"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/services/handler"
	"git.dmitriygnatenko.ru/dima/dmitriygnatenko-v2/internal/services/i18n"
)

func TagHandler(
	tagRepository TagRepository,
) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		tags, err := tagRepository.GetAll(fctx.Context())
		if err != nil {
			logger.Error(fctx.Context(), err.Error())
			return err
		}

		return fctx.Render("admin/tag", fiber.Map{
			"tags":    mapper.ToTagsList(tags),
			"section": "tag",
		}, "admin/_layout")
	}
}

func AddTagHandler(
	tagRepository TagRepository,
) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		ctx := fctx.Context()
		lang := mapper.LanguageFromContext(fctx)
		var form models.TagForm
		var validate = validator.New()
		validateErrors := make(map[string]string)

		trans, err := helper.GetDefaultTranslator(validate)
		if err != nil {
			logger.Error(ctx, err.Error())
			return err
		}

		if fctx.Method() == fiber.MethodPost {
			if err = fctx.BodyParser(&form); err != nil {
				logger.Info(ctx, err.Error())
				return err
			}

			if err = validate.Struct(form); err != nil {
				validateErrors = helper.FormatValidateErrors(err, trans)
			}

			if res, _ := tagRepository.GetByURL(ctx, form.URL); res != nil {
				validateErrors["TagForm.URL"] = i18n.T(lang, "err_tag_exists")
			}

			if len(validateErrors) == 0 {
				err = tagRepository.Add(ctx, mapper.ToTagModel(form))
				if err != nil {
					logger.Error(ctx, err.Error())
					return err
				}

				return fctx.Redirect("/admin/tag")
			}
		}

		return fctx.Render("admin/tag_edit", fiber.Map{
			"form":    form,
			"errors":  validateErrors,
			"section": "tag",
			"title":   i18n.T(lang, "admin_add_tag_title"),
		}, "admin/_layout")
	}
}

func EditTagHandler(
	tagRepository TagRepository,
	cacheService CacheService,
) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		ctx := fctx.Context()
		lang := mapper.LanguageFromContext(fctx)
		var validate = validator.New()
		validateErrors := make(map[string]string)

		trans, err := helper.GetDefaultTranslator(validate)
		if err != nil {
			logger.Error(ctx, err.Error())
			return err
		}

		id, err := request.ConvertToUint64(fctx, "id")
		if err != nil {
			logger.Error(ctx, err.Error())
			return err
		}

		tag, err := tagRepository.GetByID(ctx, id)
		if err != nil {
			logger.Error(ctx, err.Error())
			return err
		}

		form := models.TagForm{
			ID:  id,
			Tag: tag.Tag,
			URL: tag.URL,
		}

		if fctx.Method() == fiber.MethodPost {
			if err = fctx.BodyParser(&form); err != nil {
				logger.Info(ctx, err.Error())
				return err
			}

			if err = validate.Struct(form); err != nil {
				validateErrors = helper.FormatValidateErrors(err, trans)
			}

			if res, _ := tagRepository.GetByURL(ctx, form.URL); res != nil {
				if res.ID != id {
					validateErrors["TagForm.URL"] = i18n.T(lang, "err_tag_exists")
				}
			}

			if len(validateErrors) == 0 {
				err = tagRepository.Update(ctx, mapper.ToTagModel(form))
				if err != nil {
					logger.Error(ctx, err.Error())
					return err
				}

				cacheService.Delete(handler.TagCacheKey + string(i18n.Ru) + tag.URL)

				return fctx.Redirect("/admin/tag")
			}
		}

		return fctx.Render("admin/tag_edit", fiber.Map{
			"form":    form,
			"errors":  validateErrors,
			"section": "tag",
			"title":   i18n.T(lang, "admin_edit_tag_title"),
		}, "admin/_layout")
	}
}

func DeleteTagHandler(
	tagRepository TagRepository,
	cacheService CacheService,
) fiber.Handler {
	return func(fctx *fiber.Ctx) error {
		ctx := fctx.Context()
		id, err := request.ConvertToUint64(fctx, "id")
		if err != nil {
			logger.Error(ctx, err.Error())
			return err
		}

		tag, err := tagRepository.GetByID(ctx, id)
		if err != nil {
			logger.Error(ctx, err.Error())
			return err
		}

		used, err := tagRepository.IsUsed(ctx, id)
		if err != nil {
			logger.Error(ctx, err.Error())
			return err
		}

		if fctx.Method() == fiber.MethodPost {
			err = tagRepository.Delete(ctx, id)
			if err != nil {
				logger.Error(ctx, err.Error())
				return err
			}

			cacheService.Delete(handler.TagCacheKey + string(i18n.Ru) + tag.URL)

			return fctx.Redirect("/admin/tag")
		}

		return fctx.Render("admin/tag_delete", fiber.Map{
			"tag":     tag.Tag,
			"used":    used,
			"section": "tag",
		}, "admin/_layout")
	}
}
